@@ -1,6 +1,6 @@
 // DasherModel.cpp
 //
-// Copyright (c) 2007 The Dasher Team
+// Copyright (c) 2008 The Dasher Team
 //
 // This file is part of Dasher.
 //
@@ -21,6 +21,7 @@
 #include "../Common/Common.h"
 
 #include <iostream>
+#include <cstring>
 #include "../Common/Random.h"
 #include "DasherModel.h"
 #include "DasherView.h"
@@ -33,7 +34,7 @@
 #include "LanguageModelling/DictLanguageModel.h"
 #include "LanguageModelling/MixtureLanguageModel.h"
 #include "NodeCreationManager.h"
-//#include "DasherGameMode.h"
+#include "DasherGameMode.h"
 #include "AlphabetManager.h"
 
 using namespace Dasher;
@@ -62,7 +63,7 @@ CDasherModel::CDasherModel(CEventHandler *pEventHandler,
   m_pNodeCreationManager = pNCManager;
   m_pDasherInterface = pDasherInterface;
 
-  //  m_bGameMode = GetBoolParameter(BP_GAME_MODE);
+  m_bGameMode = GetBoolParameter(BP_GAME_MODE);
   m_iOffset = iOffset; // TODO: Set through build routine
 
   DASHER_ASSERT(m_pNodeCreationManager != NULL);
@@ -135,7 +136,7 @@ void CDasherModel::HandleEvent(Dasher::CEvent *pEvent) {
 	m_iStartTime = 1;
       break;
     case BP_GAME_MODE:
-      //      m_bGameMode = GetBoolParameter(BP_GAME_MODE);
+      m_bGameMode = GetBoolParameter(BP_GAME_MODE);
       // Maybe reload something here to begin game mode?
       break;
     default:
@@ -177,11 +178,6 @@ void CDasherModel::Make_root(CDasherNode *pNewRoot) {
     delete oldroots[0];
     oldroots.pop_front();
   }
-
-//   for(int i(0); i < oldroots.size(); ++i)
-//     std::cout << oldroots[i]->GetFlag(NF_SUBNODE) << " ";
-
-//  std::cout << std::endl;
   
   m_Root = pNewRoot;
 
@@ -218,8 +214,6 @@ void CDasherModel::RecursiveMakeRoot(CDasherNode *pNewRoot) {
 void CDasherModel::RebuildAroundNode(CDasherNode *pNode) {
   DASHER_ASSERT(pNode != NULL);
 
-  CDasherNode *pNodeOrig = pNode;
-
   while(pNode->GetFlag(NF_SUBNODE))
     pNode = pNode->Parent();
 
@@ -279,8 +273,8 @@ void CDasherModel::Reparent_root(int lower, int upper) {
     if(!(pNewRoot->GetFlag(NF_SUBNODE)) && 
        (((myint((GetLongParameter(LP_NORMALIZATION) - upper)) / static_cast<double>(iWidth)) >
 	 (m_Rootmax_max - m_Rootmax)/static_cast<double>(iRootWidth)) || 
-       ((myint(lower) / static_cast<double>(iWidth)) > 
-	(m_Rootmin - m_Rootmin_min) / static_cast<double>(iRootWidth)))) {
+	((myint(lower) / static_cast<double>(iWidth)) > 
+	 (m_Rootmin - m_Rootmin_min) / static_cast<double>(iRootWidth)))) {
       pNewRoot->OrphanChild(m_Root);
       delete pNewRoot;
       return;
@@ -425,6 +419,19 @@ void CDasherModel::Get_new_root_coords(myint Mousex, myint Mousey, myint &iNewMi
   int iTargetMin(Mousey - ((myint)iMaxY * Mousex) / (2 * (myint)iOX));
   int iTargetMax(Mousey + ((myint)iMaxY * Mousex) / (2 * (myint)iOY));
 
+  // If (0,iMaxY) = (iTargetMin,iTargetMax), the "zoom factor" is 1, we
+  // don't want to do anything.
+  if (0 == iTargetMin && iMaxY == iTargetMax)
+    {
+      iNewMin = m_Rootmin;
+      iNewMax = m_Rootmax;
+      return;
+    }
+
+  // Compute the point C which is thus the center of expansion
+  // (it divides iTargetMin-Max into the same proportions at it divides 0-iMaxY)
+  const dasherint C = (iTargetMin * iMaxY) / (iTargetMin + iMaxY - iTargetMax);
+
   // iSteps is the number of update steps we need to get the point
   // under the cursor over to the cross hair. Calculated in order to
   // keep a constant bit-rate.
@@ -459,8 +466,9 @@ void CDasherModel::Get_new_root_coords(myint Mousex, myint Mousey, myint &iNewMi
     iTargetMax = iNewTargetMax;
   }
 
-  iNewMin = (((m_Rootmin - iTargetMin) * (myint)GetLongParameter(LP_MAX_Y)) / (iTargetMax - iTargetMin));
-  iNewMax = (((m_Rootmax - iTargetMax) * (myint)GetLongParameter(LP_MAX_Y)) / (iTargetMax - iTargetMin) + (myint)GetLongParameter(LP_MAX_Y));
+  //...and go there in one step, as we've already "stepped" the desired viewport
+  iNewMin = ((m_Rootmin - C) * iMaxY) / (iTargetMax - iTargetMin) + C;
+  iNewMax = ((m_Rootmax - C) * iMaxY) / (iTargetMax - iTargetMin) + C;
 }
 
 bool CDasherModel::UpdatePosition(myint miMousex, myint miMousey, unsigned long iTime, Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted) {
@@ -471,7 +479,7 @@ bool CDasherModel::UpdatePosition(myint miMousex, myint miMousey, unsigned long
     *pNumDeleted = 0;
 
   if(GetBoolParameter(BP_DASHER_PAUSED) && (m_deGotoQueue.size() == 0))
-     return false;
+    return false;
 
   // Find out the current node under the crosshair
   CDasherNode *old_under_cross=Get_node_under_crosshair();
@@ -515,9 +523,9 @@ bool CDasherModel::UpdatePosition(myint miMousex, myint miMousey, unsigned long
 void CDasherModel::NewFrame(unsigned long Time) {
   m_fr.NewFrame(Time);
   ///GAME MODE TEMP///Pass new frame events onto our teacher
-  //  CDasherGameMode* pTeacher = CDasherGameMode::GetTeacher();
-  // if(m_bGameMode && pTeacher)
-  //  pTeacher->NewFrame(Time);
+  GameMode::CDasherGameMode* pTeacher = GameMode::CDasherGameMode::GetTeacher();
+  if(m_bGameMode && pTeacher)
+    pTeacher->NewFrame(Time);
 }
 
 void CDasherModel::OldPush(myint iMousex, myint iMousey) {
@@ -561,9 +569,9 @@ void CDasherModel::OldPush(myint iMousex, myint iMousey) {
       int iRandom = RandomInt();
       
       if(Framerate() > 8) {
-	      // add some noise and push another node
+	// add some noise and push another node
       	CDasherNode *pRight = Get_node_under_mouse(50, iMousey + iRandom % 500 - 250);
-	      Push_Node(pRight);
+	Push_Node(pRight);
       }
       
       iRandom = RandomInt();
@@ -587,9 +595,12 @@ void CDasherModel::RecursiveOutput(CDasherNode *pNode, Dasher::VECTOR_SYMBOL_PRO
   pNode->SetFlag(NF_SEEN, true);
   pNode->m_pNodeManager->Output(pNode, pAdded, GetLongParameter(LP_NORMALIZATION));
 
-  //  if(m_bGameMode)
-  // if(pNode->GetFlag(NF_END_GAME))
-  //   CDasherGameMode::GetTeacher()->SentenceFinished();
+  // If the node we are outputting is the last one in a game target sentence, then
+  // notify the game mode teacher.
+  if(m_bGameMode)
+    if(pNode->GetFlag(NF_END_GAME))
+      GameMode::CDasherGameMode::GetTeacher()->SentenceFinished();
+
 }
 
 void CDasherModel::NewGoTo(myint newRootmin, myint newRootmax, Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted) {
@@ -658,24 +669,24 @@ bool CDasherModel::DeleteCharacters(CDasherNode *newnode, CDasherNode *oldnode,
   if(newnode->GetFlag(NF_SEEN)) {
     // TODO: I really cannot believe that  the following could ever have worked.
     
-  //   if(oldnode->Parent() == newnode) {
-//       std::cout << "DCA" << std::endl;
-//       oldnode->m_pNodeManager->Undo(oldnode);
-//       oldnode->Parent()->m_pNodeManager->Enter(oldnode->Parent());
-//       if (pNumDeleted != NULL)
-//         (*pNumDeleted)++;
-//       oldnode->SetFlag(NF_SEEN, false);
-//       return true;
-//     }
-//     if(DeleteCharacters(newnode, oldnode->Parent(), pNumDeleted) == true) {
-//       std::cout << "DCB" << std::endl;
-//       oldnode->m_pNodeManager->Undo(oldnode);
-//       oldnode->Parent()->m_pNodeManager->Enter(oldnode->Parent());
-//       if (pNumDeleted != NULL)
-// 	(*pNumDeleted)++;
-//       oldnode->SetFlag(NF_SEEN, false);
-//       return true;
-//     }
+    //   if(oldnode->Parent() == newnode) {
+    //       std::cout << "DCA" << std::endl;
+    //       oldnode->m_pNodeManager->Undo(oldnode);
+    //       oldnode->Parent()->m_pNodeManager->Enter(oldnode->Parent());
+    //       if (pNumDeleted != NULL)
+    //         (*pNumDeleted)++;
+    //       oldnode->SetFlag(NF_SEEN, false);
+    //       return true;
+    //     }
+    //     if(DeleteCharacters(newnode, oldnode->Parent(), pNumDeleted) == true) {
+    //       std::cout << "DCB" << std::endl;
+    //       oldnode->m_pNodeManager->Undo(oldnode);
+    //       oldnode->Parent()->m_pNodeManager->Enter(oldnode->Parent());
+    //       if (pNumDeleted != NULL)
+    // 	(*pNumDeleted)++;
+    //       oldnode->SetFlag(NF_SEEN, false);
+    //       return true;
+    //     }
 
     oldnode->m_pNodeManager->Undo(oldnode);
     oldnode->Parent()->m_pNodeManager->Enter(oldnode->Parent());
@@ -725,10 +736,9 @@ bool CDasherModel::DeleteCharacters(CDasherNode *newnode, CDasherNode *oldnode,
 
 void CDasherModel::Push_Node(CDasherNode *pNode) {
   DASHER_ASSERT(pNode != NULL);
-
   // TODO: Fix this and make an assertion again
   if(pNode->GetFlag(NF_SUBNODE))
-      return;
+    return;
 
   // TODO: Is NF_ALLCHILDREN any more useful/efficient than reading the map size?
   
@@ -757,56 +767,73 @@ void CDasherModel::Push_Node(CDasherNode *pNode) {
   ///GAME MODE TEMP///////////
   // If we are in GameMode, then we do a bit of cooperation with the teacher object when we create
   // new children.
-//   CDasherGameMode* pTeacher = CDasherGameMode::GetTeacher();
-//   if(m_bGameMode && pNode->GetFlag(NF_GAME) && pTeacher )
-//     {
-//       std::string strTarget;
-//       CAlphabetManager::SAlphabetData * pAlphabetData =
-// 	static_cast<CAlphabetManager::SAlphabetData *>(pNode->m_pUserData);
-//       strTarget = pTeacher->GetSymbolAtOffset(pAlphabetData->iOffset+1);
-//       CDasherNode::ChildMap::iterator i, j;
-//       /////////////FIX/////////////////
-//       if(strTarget == "GameEnd")
-// 	{
-// 	  pNode->SetFlag(NF_END_GAME, true);
-// 	  goto multibreak;
-// 	}
-//       for(i = pNode->Children().begin(); i != pNode->Children().end(); i++)
-// 	{
-// 	  if((*i)->GetDisplayInfo()->strDisplayText == "Control") continue;
 
-// 	  if((*i)->GetFlag(NF_SUBNODE))
-// 	    {
-// 	      for(j = (*i)->Children().begin(); j != (*i)->Children().end(); j++)
-// 		{
-// 		  std::string strNode;
-// 		  CDasherNode * pTempNode = (*j);
-// 		  pAlphabetData = static_cast<CAlphabetManager::SAlphabetData *>(pTempNode->m_pUserData);
-// 		  strNode = m_pNodeCreationManager->GetAlphabet()->GetText(pAlphabetData->iSymbol);
-// 		  if(strNode == strTarget)
-// 		    {
-// 		      (*j)->SetFlag(NF_GAME, true);
-// 		      (*i)->SetFlag(NF_GAME, true);
-// 		      goto multibreak;
-// 		    }
-// 		}
-// 	    }
-// 	  else
-// 	    {
-// 	      std::string strNode;
-// 	      CDasherNode * pTempNode = (*i);
-// 	      pAlphabetData = static_cast<CAlphabetManager::SAlphabetData *>(pTempNode->m_pUserData);
-// 	      strNode = m_pNodeCreationManager->GetAlphabet()->GetText(pAlphabetData->iSymbol);
-// 	      if(strNode == strTarget)
-// 		{
-// 		  (*i)->SetFlag(NF_GAME, true);
-// 		  goto multibreak;
-// 		}	 
-// 	    }
-// 	}
-//     multibreak:
-//       true;
-//     }
+  GameMode::CDasherGameMode* pTeacher = GameMode::CDasherGameMode::GetTeacher();
+  if(m_bGameMode && pNode->GetFlag(NF_GAME) && pTeacher )
+    {
+      std::string strTargetUtf8Char;
+      
+      CAlphabetManager::SAlphabetData * pAlphabetData =
+	static_cast<CAlphabetManager::SAlphabetData *>(pNode->m_pUserData);
+      
+      strTargetUtf8Char = pTeacher->GetSymbolAtOffset(pAlphabetData->iOffset+1);
+      
+      CDasherNode::ChildMap::iterator i, j;
+      // Check if this is the last node in the sentence...
+      if(strTargetUtf8Char == "GameEnd")
+	{
+	  pNode->SetFlag(NF_END_GAME, true);
+	  goto multibreak;
+	}
+      // ...if it is not then find which child is next in line.
+      for(i = pNode->Children().begin(); i != pNode->Children().end(); i++)
+	{
+	  // Only look for children who are the same type of node as pNode is (i.e. alphabet)
+	  if((*i)->m_pNodeManager != pNode->m_pNodeManager) continue;
+	  
+	  // Look at the children of the group nodes while we search
+	  if((*i)->GetFlag(NF_SUBNODE))
+	    {
+	      for(j = (*i)->Children().begin(); j != (*i)->Children().end(); j++)
+		{
+		  std::string strNodeUtf8Char;
+		  CDasherNode * pTempNode = (*j);
+		  
+		  pAlphabetData = static_cast<CAlphabetManager::SAlphabetData *>(pTempNode->m_pUserData);
+		  strNodeUtf8Char = m_pNodeCreationManager->GetAlphabet()->GetText(pAlphabetData->iSymbol);
+		  
+		  if(strNodeUtf8Char == strTargetUtf8Char)
+		    {
+		      (*j)->SetFlag(NF_GAME, true);
+		      (*i)->SetFlag(NF_GAME, true);
+		      goto multibreak;
+		    }
+		}
+	    }
+	  else
+	    {
+	      std::string strNodeUtf8Char;
+	      CDasherNode * pTempNode = (*i);
+
+	      pAlphabetData = static_cast<CAlphabetManager::SAlphabetData *>(pTempNode->m_pUserData);
+	      strNodeUtf8Char = m_pNodeCreationManager->GetAlphabet()->GetText(pAlphabetData->iSymbol);
+
+	      if(strNodeUtf8Char == strTargetUtf8Char)
+		{
+		  (*i)->SetFlag(NF_GAME, true);
+		  goto multibreak;
+		}	 
+	    }
+	}
+      // If this line runs then we have a target character which is not in our current alphabet.
+      // So let's give up!
+      pNode->SetFlag(NF_END_GAME, true);
+      
+    multibreak:
+      ; // A lonely semicolon is the null statement - it does nothing.
+    }
+  ////////////////////////////
+  
 
 }
 
@@ -829,7 +856,10 @@ void CDasherModel::Recursive_Push_Node(CDasherNode *pNode, int iDepth) {
   }
 }
 
-bool CDasherModel::RenderToView(CDasherView *pView, bool bRedrawDisplay) {
+bool CDasherModel::RenderToView(CDasherView *pView, bool bRedrawDisplay, SLockData *pLockData) {
+
+  if(pLockData)
+    std::cout << "Rendering locked: " << pLockData->strDisplay << std::endl;
 
   DASHER_ASSERT(pView != NULL);
   DASHER_ASSERT(m_Root != NULL);
@@ -849,9 +879,9 @@ bool CDasherModel::RenderToView(CDasherView *pView, bool bRedrawDisplay) {
   
 
   /////////GAME MODE TEMP//////////////
-//   if(m_bGameMode)
-//     if(CDasherGameMode* pTeacher = CDasherGameMode::GetTeacher())
-//       pTeacher->SetTargetY(vGameTargetY);
+  if(m_bGameMode)
+    if(GameMode::CDasherGameMode* pTeacher = GameMode::CDasherGameMode::GetTeacher())
+      pTeacher->SetTargetY(vGameTargetY);
   //////////////////////////////////////
 
 
@@ -910,7 +940,6 @@ bool CDasherModel::CheckForNewRoot(CDasherView *pView) {
   CDasherNode *pOldNode = Get_node_under_crosshair();
 
   CDasherNode *root(m_Root);
-  CDasherNode::ChildMap & children = m_Root->Children();
 
   if(!(m_Root->GetFlag(NF_SUPER))) {
     Reparent_root(root->Lbnd(), root->Hbnd());
@@ -928,8 +957,7 @@ bool CDasherModel::CheckForNewRoot(CDasherView *pView) {
     // TODO: I think this if statement is reduncdent, return value of above is always equal to bFound
     // not true - pconlon
     ////GAME MODE TEMP - only change the root if it is on the game path/////////
-//       if(bFound && (!m_bGameMode || pNewRoot->GetFlag(NF_GAME)))
-    if(bFound) {
+    if(bFound && (!m_bGameMode || pNewRoot->GetFlag(NF_GAME))) {
       m_Root->DeleteNephews(pNewRoot);
       RecursiveMakeRoot(pNewRoot);
     }
@@ -946,57 +974,81 @@ bool CDasherModel::CheckForNewRoot(CDasherView *pView) {
   return false;
 }
 
-double CDasherModel::CorrectionFactor(int dasherx, int dashery) {
-  double dX = 1 - dasherx/2048.0;
-  double dY = dashery/2048.0 - 1;
 
-  double dR = sqrt(pow(dX, 2.0) + pow(dY, 2.0));
+void CDasherModel::ScheduleZoom(dasherint X, dasherint Y, int iMaxZoom)
+{
+  // XXX PRLW Are these parameters really long?
+  const int nsteps = GetLongParameter(LP_ZOOMSTEPS);
+  const int safety = GetLongParameter(LP_S); // over safety_denom gives %
+  const int safety_denom = 1024;
+  const int ymax = GetLongParameter(LP_MAX_Y);
+  const int scale = ymax; // (0,1) -> (ymin=0,ymax)
 
-  if(fabs(dX) < 0.1)
-    return dR * (1 + dX /2.0+ pow(dX, 2.0) / 3.0 + pow(dX, 3.0) / 4.0 + pow(dX, 4.0) / 5.0);
-  else
-    return -dR * log(1 - dX) / dX;
-}
+  // (X,Y) is mouse position in dasher coordinates
+  // Prevent clicking too far to the right => y1 <> y2 see below
+  if (X < 2) X = 2;
 
-void CDasherModel::ScheduleZoom(dasherint iDasherX, dasherint iDasherY) {
+  // Lines with gradient +/- 1 passing through (X,Y) intersect y-axis at
+  dasherint y1 = Y - X;     // y =  x + (Y - X)
+  dasherint y2 = Y + X;     // y = -x + (Y + X)
 
-  // Prevent clicking too far to the right
-  if (iDasherX < 2) { iDasherX = 2; }
-  
-  // TODO: Document this 
-  // double dCFactor = CorrectionFactor(iDasherX, iDasherY);
-  // int iSteps = static_cast<int>(GetLongParameter(LP_ZOOMSTEPS) * dCFactor);
-  
-  int iSteps = GetLongParameter(LP_ZOOMSTEPS);
-  
-  myint iC;
-  
-  if(iDasherX < 2048)
-    iC = ((iDasherY - 2048) * iDasherX) / 2048 + iDasherY;
-  else
-    iC = iDasherY - ((iDasherY - 2048) * iDasherX) / 2048;
-  
-  myint iTarget1 = iDasherY - iDasherX;
-  myint iTarget2 = iDasherY + iDasherX;
-  
-  double dZ = 4096 / static_cast<double>(iTarget2 - iTarget1);
-  double dTau = 1.0/log(dZ);
-  
+  // The maximum linear region of the y-axis is from 5% - 95% of ymax
+  // 5% and 95% come from dY3, dY2 in CDasherViewSquare::Cymap::Cymap,
+  // and non-linearity between dasher coords m_Y3 to m_Y2 in
+  // CDasherViewSquare::Cymap::map
+  // (It is good to test with Y1 = 0, Y2 = ymax.)
+  // XXX PRLW But m_Y3, m_Y2 are out of scope...
+  const dasherint Y1 = ( 5 * scale) / 100;  // m_Y3;
+  const dasherint Y2 = (95 * scale) / 100;  // m_Y2;
+
+  // So, want to zoom (y1 - safety/2, y2 + safety/2) -> (Y1, Y2)
+  // Adjust y1, y2 for safety margin
+  dasherint ds = (safety * scale) / (2 * safety_denom);
+  y1 -= ds;
+  y2 += ds;
+
+  // If (y1,y2) = (Y1,Y2), the "zoom factor" is 1, we don't want to do
+  // anything.
+  if (y1 == Y1 && y2 == Y2) return;
+
+  // There is a point C on the y-axis such the ratios (y1-C):(Y1-C) and
+  // (y2-C):(Y2-C) are equal. (Obvious when drawn on separate parallel axes.)
+  // (Y2 - Y1 is large.)
+  dasherint C = (y1 * Y2 - y2 * Y1) / (y1 + Y2 - y2 - Y1);
+
+  // So another point r's zoomed y coordinate R, has the same ratio (r-C):(R-C)
+  // Rename for readability.
+  const dasherint rmin = m_Rootmin;
+  const dasherint rmax = m_Rootmax;
+  dasherint Rmin, Rmax;
+  if (y1 != C) {
+      Rmin = ((rmin - C) * (Y1 - C)) / (y1 - C) + C;
+      Rmax = ((rmax - C) * (Y1 - C)) / (y1 - C) + C;
+  } else if (y2 != C) {
+      Rmin = ((rmin - C) * (Y2 - C)) / (y2 - C) + C;
+      Rmax = ((rmax - C) * (Y2 - C)) / (y2 - C) + C;
+  } else { // implies y1 = y2
+      std::cerr << "Impossible geometry in CDasherModel::ScheduleZoom\n";
+  }
+
+  // iMaxZoom seems to be in tenths
+  if (iMaxZoom != 0 && 10 * (Rmax - Rmin) > iMaxZoom * (rmax - rmin)) {
+      Rmin = ((rmin - C) * iMaxZoom) / 10 + C;
+      Rmax = ((rmax - C) * iMaxZoom) / 10 + C;
+  }
+
+  // sNewItem seems to contain a list of root{min,max} for the frames of the
+  // zoom, so split r -> R into n steps, with accurate R
   m_deGotoQueue.clear();
-  
-  for(int s = 0; s < iSteps; ++s) {
-    double dFraction = s / static_cast<double>(iSteps - 1);
-    
-    SGotoItem sNewItem;
-    sNewItem.iN1 = (m_Rootmin - iC) * exp(dFraction/dTau) + iC;
-    sNewItem.iN2 = (m_Rootmax - iC) * exp(dFraction/dTau) + iC;
-    
-    m_deGotoQueue.push_back(sNewItem);
+  for (int s = nsteps - 1; s >= 0; --s) {
+      SGotoItem sNewItem;
+      sNewItem.iN1 = Rmin - (s * (Rmin - rmin)) / nsteps;
+      sNewItem.iN2 = Rmax - (s * (Rmax - rmax)) / nsteps;
+      m_deGotoQueue.push_back(sNewItem);
   }
-  
-  return;
 }
 
+
 void CDasherModel::Offset(int iOffset) {
   m_Rootmin += iOffset;
   m_Rootmax += iOffset;
