@@ -1,6 +1,5 @@
-
-#ifndef _WIN32
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
 #endif 
 
 #include "LanguageModelling/LanguageModel.h"
@@ -12,7 +11,7 @@
 
 #include <fstream>
 #include <iostream>
-
+#include <cstdlib>
 
 
 using namespace Dasher;
